#!/usr/bin/perl
#this program converts civs modpacks to their freeciv counterparts
#Copyright (c) 2003, 2004 John Bauman

#Permission is hereby granted, free of charge, to any person obtaining a copy 
#of this software and associated documentation files (the "Software"), to deal 
#in the Software without restriction, including without limitation the rights 
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
#copies of the Software, and to permit persons to whom the Software is furnished 
#to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all 
#copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
#INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
#PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
#HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION 
#OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
#SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


open (IFILE, "rules.txt") || die "There needs to be a rules.txt in the current directory.";

while (chomp ($x = <IFILE>) && !($x eq "\@COSMIC")) {}
$cosmnum = 0;
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  ($cosm[$cosmnum], $comment) = split(/;/,$x);
  $cosm[$cosmnum] =~ s/ //g;
  $cosmnum++;
}



while (chomp ($x = <IFILE>) && !($x eq "\@CIVILIZE")) {}

$technum = 0;
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  ($name[$technum], $blah1, $blah4, $req1[$technum], $req2[$technum], $blah2, $blah3) = split(/,/,$x);
  ($blah2, $n) = split(/\;/, $blah3);
  $req1[$technum] =~ s/ //g;
  $req2[$technum] =~ s/ //g;
  $n =~ / *([A-Za-z0-9]+) */;
  $m = $1;
  $short{$m} = $name[$technum];
  $sname[$technum] = $m;

  #Civ2 annoyingly special-cases these
  #are they based on the actual name?
  $flags[$technum] = ""; 
  $flags[$technum] = "Population_Pollution_Inc" if ($m eq "Aut");
  $flags[$technum] = "Bridge" if ($m eq "Bri");
  $flags[$technum] = "Fortress" if ($m eq "Cst");
  $flags[$technum] = "Trade_Revenue_Reduce" if ($m eq "Fli");
  $flags[$technum] = "Population_Pollution_Inc" if ($m eq "Ind");
  $flags[$technum] = "Population_Pollution_Inc" if ($m eq "MP");
  $flags[$technum] = "Reduce_Trireme_Loss2" if ($m eq "Nav");
  $flags[$technum] = "Boat_Fast" if ($m eq "NP");
  $flags[$technum] = "Bonus_Tech" if ($m eq "Phi");
  $flags[$technum] = "Population_Pollution_Inc" if ($m eq "Pla");
  $flags[$technum] = "Airbase" if ($m eq "Rad");
  $flags[$technum] = "Railroad\", \"Trade_Revenue_Reduce" if ($m eq "RR");
  $flags[$technum] = "Farmland" if ($m eq "Rfg");
  $flags[$technum] = "Reduce_Trireme_Loss1" if ($m eq "Sea");
  $technum++;
}
$short{'nil'} = "None";
$short{'no'} = "Never";

open FRET, ">","techs.ruleset";
print FRET "[datafile]\n";
print FRET "description=\"Civ2 modpack techs\"\n";
print FRET "options=\"1.9\"\n\n";

for ($i = 0;$i < $technum; $i++) {
  $name2 = $name[$i];
  $name2 =~ tr/[A-Z] /[a-z]_/;
  print FRET "[advance_$name2]\n";
  print FRET "name     = _(\"$name[$i]\")\n";
  print FRET "req1     = \"$short{$req1[$i]}\"\n";
  print FRET "req2     = \"$short{$req2[$i]}\"\n";
  print FRET "flags    = \"$flags[$i]\"\n";
  print FRET "\n"; 
}
close FRET;




while (chomp ($x = <IFILE>) && !($x eq "\@GOVERNMENTS")) {}

$gnum = 0;
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  ($gname[$gnum],$gmtitle[$gnum],$gftitle[$gnum]) = split(/,/, $x);
  $gmtitle[$gnum] =~ s/^ +//;
  $gftitle[$gnum] =~ s/^ +//;
  $gnum++;
}

open (GTM, "gtemp.txt") || die "There needs to be a gtemp.txt in the current directory.";
open GOUT, ">", "governments.ruleset";

$sgov = -1;
while (<GTM>) {
  chomp;
  $sgov++ if (/##newgov/);
  s/##newgov//;
  s/##gov(\d)##/$gname[$1]/g;
  s/##tech-([A-Za-z0-9]*)##/$short{$1}/g;
  s/##mname##/$gmtitle[$sgov]/g;
  s/##fname##/$gftitle[$sgov]/g;
  s/##cosm(\d*)##/$cosm[$1]/g;
  print GOUT "$_\n";
}
close GOUT;
close GTM;




seek IFILE, 0, 0;
while (chomp ($x = <IFILE>) && !($x eq "\@IMPROVE")) {}

$impnum = 0;
$ireal = 0; #used to keep track of deletions
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  if ($ireal eq 0) {#freeciv has no "Nothing"
    $ireal++;
    next;
  }
  ($bname[$impnum], $cost[$impnum], $upkeep[$impnum], $breq[$impnum], $blank) = split(/,/,$x);
  $bname[$impnum] =~ s/\(//; $bname[$impnum] =~ s/\)//;
  $cost[$impnum] =~ s/ //g; $cost[$impnum] *= 10;
  $upkeep[$impnum] =~ s/ //g;
  $breq[$impnum] =~ s/ //g; $breq[$impnum] = $short{$breq[$impnum]};
  $exptech[$impnum] = "";
  $won[$impnum] = 0;
  $impnum++;
}

while (chomp ($x = <IFILE>) && !($x eq "\@ENDWONDER")) {}
$i = $impnum - 28;
while (chomp ($x = <IFILE>) && !($x eq "")) {
  $x =~ /(..+),/;
  $exptech[$i] = $short{$1};
  $won[$i] = 1;
  $i++;
}
sub swap($$) {
  my $a = shift;
  my $b = shift;
  return ($b, $a);
}
#civ2 doesn't include a lot of things in its definition, so I'm using a template
open(BT,"btemp.txt") || die "There needs to be a btemp.txt in the current directory.";;
open FBL,">","buildings.ruleset";
while (chomp ($x = <BT>) && !($x eq "##endmove") ) {#move around the locations
  ($sa, $sb) = split(/-/,$x);
  ($bname[$sa], $bname[$sb]) = swap($bname[$sa], $bname[$sb]);
  ($cost[$sa], $cost[$sb]) = swap($cost[$sa], $cost[$sb]);
  ($upkeep[$sa], $upkeep[$sb]) = swap($upkeep[$sa], $upkeep[$sb]);
  ($breq[$sa], $breq[$sb]) = swap($breq[$sa], $breq[$sb]);
  ($exptech[$sa], $exptech[$sb]) = swap($exptech[$sa], $exptech[$sb]);
  ($won[$sa], $won[$sb]) = swap($won[$sa], $won[$sb]);
}
$ci = -1;
while (<BT>) {
  chomp;
  $ci++ if (/##\[/);
  $ci-- if (/##\[building_barracks_i/);
  if (/barracks\]/) {
    $exptech[$ci] = $short{'Gun'};
  } elsif(/^\[building_barracks_ii\]/) {
    $breq[$ci] = $short{'Gun'};
    $exptech[$ci] = $short{'Mob'};
    $bname[$ci] .= " II";
  } elsif(/^\[building_barracks_iii\]/) {
    $breq[$ci] = $short{'Mob'};
    $exptech[$ci] = "None";
    $bname[$ci] .= "I";
  }
  s/^##.+//;
  s/##name##/$bname[$ci]/;
  s/##tech##/$breq[$ci]/;
  $ob = ($exptech[$ci] eq "" ? "None" : $exptech[$ci]) if (/##obs##/);
  s/##obs##/$ob/;
  s/##won##/$won[$ci]/;
  s/##build##/$cost[$ci]/;
  s/##upk##/$upkeep[$ci]/;
  s/##bul(\d*)##/$bname[$1]/g;
  s/##cosm(\d*)##/$cosm[$1]/g;
  s/##tech-([A-Za-z0-9]*)##/$short{$1}/g;
  s/##gov(\d)##/$gname[$1]/g;
  print FBL "$_\n";
}
close FBL;
close BT;








while (chomp ($x = <IFILE>) && !($x eq "\@UNITS")) {}

$unum = 0;
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  my @unitp = split(/,/, $x);
  ($unitp[13],$comments) = split(/;/,$unitp[13]);
  for ($i = 1; $i < 14; $i++) {
    $unitp[$i] =~ s/ //g;   
  }
  $unitp[3] =~ s/\.//;
  $unitp[5] =~ s/a//; 
  $unitp[6] =~ s/d//; 
  $unitp[7] =~ s/h//;
  $unitp[8] =~ s/f//;
  $unitp[1] = $short{$unitp[1]};
  $unitp[12] = $short{$unitp[12]};
  $unitp[2] =~ s/0/Land/;
  $unitp[2] =~ s/1/Air/;
  $unitp[2] =~ s/2/Sea/;
  $unitp[9] *= 10;
  $unitp[7] *= 10;
  $ulist[$unum] = \@unitp;
  $unum++;
}
#in freeciv, units are obsoleted by units, while in civ2 they are obsoleted by techs
for ($i = 0; $i < $unum; $i++) {
  $upreq = $ulist[$i][1];
  if ($ulist[$i][1] eq "None" || $ulist[$i][1] eq "Never") {
    $ulist[$i][1] = "None";
    next;
  }
  for ($j = 0; $j < $unum; $j++) {
    if ($ulist[$i][1] eq $ulist[$j][12]) {
      $ulist[$i][1] = $ulist[$j][0];
      last;
    }
  }
  if ($upreq eq $ulist[$i][1]) { #Oops
    print "No unit found to obsolete: $ulist[$i][0]\n";
    $ulist[$i][1] = "None";
  }
}
sub qbstr($$) { #use to check if a number is flagged
  return substr($_[0],$_[1],1);
}
open UNITSET ,">","units.ruleset";  
open (UTMP, "utemp.txt") || die "There needs to be a utemp.txt in the current directory.";;

print UNITSET <<OUTHEAD;
[datafile]
description="Civ2 unit_type data for Freeciv (approximate)"
options="1.9"

[units_adjust]
firepower_factor=1     ; Divide hitpoints by this value. It has to be
                       ; set higher than zero.

OUTHEAD

for ($i = 0; $i < $unum; $i++) {
  $cunitx = $ulist[$i];
  @cunit = @$cunitx;
  $unames = $cunit[0];
  $unames =~ tr/[A-Z] /[a-z]_/;
  $unames =~ s/\.//g;
  $popcost = ($cunit[11] eq 5) ? 1 : 0;
  $shield = ($cunit[11] eq 6 || $cunit[11] eq 7) ? 0 : 1;
  $vision = substr($cunit[13],14,1) + 1;
  $happy = ($cunit[11] eq 5 || $cunit[11] eq 6 || $cunit[11] eq 7 ||
            ($cunit[2] eq "Air" && $cunit[4] eq 1)) ? 0 : 1;

  $uflags = "";
  $uflags .= " IgZOC" if (qbstr($cunit[13],13));
  $uflags .= " Marines" if (qbstr($cunit[13],12));
  $uflags .= " Partial_Invis Missile_Carrier No_Land_Attack" if (qbstr($cunit[13],11));
  $uflags .= " Fighter" if (qbstr($cunit[13],10));
  $uflags .= " Trireme" if (qbstr($cunit[13],9));
  $uflags .= " IgWall" if (qbstr($cunit[13],8));
  $uflags .= " Carrier" if (qbstr($cunit[13],7));
  $uflags .= " Paratroopers" if (qbstr($cunit[13],6));
  $uflags .= " IgTer" if (qbstr($cunit[13],5));
  $uflags .= " Pikemen" if (qbstr($cunit[13],4));
  $uflags .= " Fanatic" if (qbstr($cunit[13],3));
  $uflags .= " FieldUnit OneAttack Missile" if (qbstr($cunit[13],2));
  $uflags .= " AEGIS" if (qbstr($cunit[13],1));
  $uflags .= " NonMil" if ($cunit[5] eq 0);
  $uflags .= " TradeRoute HelpWonder" if ($cunit[11] eq 7);
  $uflags .= " Diplomat" if ($cunit[11] eq 6);
  $uflags .= " Settlers Airbase AddToCity Cities" if ($cunit[11] eq 5);
  $uflags .= " FieldUnit OneAttack" if ($cunit[2] eq "Air" && $cunit[4] eq 2);
  $uflags .= " Nuclear" if ($cunit[5] eq 99);
  $uflags .= " Horse" if ($cunit[3] eq 2 && $cunit[5] < 10 && $cunit[5] > 0);#how does civ2 do it?
  $uflags =~ s/^ //;
  $uflags =~ s/ /", "/g;


  $uroles = ""; #these need to be fixed
  $uroles .= " Ferryboat" if ($cunit[10] > 0 && $cunit[2] eq "Sea");
  $uroles .= " Explorer" if (qbstr($cunit[13],5) && qbstr($cunit[13],13));
  $uroles .= " Hut" if ($cunit[5] > $cunit[6] && $cunit[5] < 5);
  $uroles .= " FirstBuild" if ($cunit[5] eq 1);
  $uroles .= " Barbarian" if ($cunit[3] eq 2 && $cunit[5] <= 2);
  $uroles .= " BarbarianBuild BarbarianSea" if ($cunit[3] eq 1 && $cunit[5] <= 4 && $cunit[11] < 7);
  $uroles .= " BarbarianBoat" if ($cunit[2] eq "Sea" && !qbstr($cunit[13],9) && $cunit[10] <= 4);
  $uroles =~ s/^ //;
  $uroles =~ s/ /", "/g;

  print UNITSET <<OUTUNIT;
[unit_$unames]
name		= _("$cunit[0]")
move_type	= "$cunit[2]"
tech_req	= "$cunit[12]"
obsolete_by	= "$cunit[1]"
build_cost	= $cunit[9]
pop_cost	= $popcost
attack		= $cunit[5]
defense		= $cunit[6]
hitpoints	= $cunit[7]
firepower	= $cunit[8]
move_rate	= $cunit[3]
vision_range	= $vision
transport_cap	= $cunit[10]
fuel		= $cunit[4]
uk_happy	= $happy
uk_shield	= $shield
uk_food		= $popcost
uk_gold		= 0
flags		= "$uflags"
roles		= "$uroles"
helptext	= ""
OUTUNIT

  if (qbstr($cunit[13],6)) {
    print UNITSET <<OUTUNIT;
paratroopers_range = $cosm[19]
paratroopers_mr_req = 1
paratroopers_mr_sub = 0
OUTUNIT
  }

  $x = " ";
  while (!($x eq "") && chomp($x = <UTMP>)) {
    print UNITSET "$x\n";
  } 
  print UNITSET "\n";
}
print UNITSET <<OUTUNIT;
[unit_barbarian_leader]
name          = _("Barbarian Leader")
move_type     = "Land"
tech_req      = "None"
obsolete_by   = "$ulist[0][0]"    ; Ugly hack, to prevent anyone from building it
graphic       = "u.barbarian_leader"
graphic_alt   = "u.diplomat"
sound_move    = "m_barbarian_leader"
sound_move_alt = "m_generic"
sound_fight   = "f_barbarian_leader"
sound_fight_alt = "f_generic"
build_cost    = 40
pop_cost      = 0
attack        = 0
defense       = 0
hitpoints     = 10
firepower     = 1
move_rate     = 2
vision_range  = 1
transport_cap = 0
fuel          = 0
uk_happy      = 0
uk_shield     = 0
uk_food       = 0
uk_gold       = 0
flags         = "IgZOC", "NonMil"
roles         = "BarbarianLeader"
helptext      = _("\
When barbarian leader is killed on a tile without any defending units, \
the 100 gold ransom is paid, but only to land units and helicopters.\
")

[u_specials]
partisan_req = "$short{'Gun'}","$short{'Cmn'}"
OUTUNIT

close UNITSET;




open CITY, ">", "cities.ruleset";
print CITY <<OUTCITIES;
[datafile]
description="Cities data for Freeciv"
options="1.9"

[parameters]
add_to_size_limit  = 8		; cities >= this cannot be added to.

[citystyle_ancient]
name        = _("Ancient")
graphic     = "city.ancient"
graphic_alt = "city.european"
tech        = "None"
replaced_by = "Industrial"

[citystyle_classical]
name        = _("Classical")
graphic     = "city.classical"
graphic_alt = "-"
tech        = "None"
replaced_by = "Industrial"

[citystyle_oriental]
name        = _("Oriental")
graphic     = "city.oriental"
graphic_alt = "city.european"
tech        = "None"
replaced_by = "Industrial"

[citystyle_european]
name        = _("European")
graphic     = "city.european"
graphic_alt = "-"
tech        = "None"
replaced_by = "Industrial"

[citystyle_industrial]
name        = _("Industrial")
graphic     = "city.industrial"
graphic_alt = "-"
tech        = "$short{'RR'}"
replaced_by = "Modern"

[citystyle_modern]
name        = _("Modern")
graphic     = "city.modern"
graphic_alt = "-"
tech        = "$short{'Aut'}"
replaced_by = "PostModern"

[citystyle_postmodern]
name        = _("PostModern")
graphic     = "city.postmodern"
graphic_alt = "-"
tech        = "$short{'Sup'}"
replaced_by = "-"
OUTCITIES

close CITY;





while (chomp ($x = <IFILE>) && !($x eq "\@TERRAIN")) {}

for ($i = 0; $i < 11; $i++) {#load main terrains
  chomp ($x = <IFILE>);
  my @tp = split(/,/,$x);
  ($a, $b) = split(/;/,$tp[15]);
  $tp[15] = $b; 
  for ($j = 1; $j < 16; $j++) {
    $tp[$j] =~ s/ //g;
  }
  $tshort{$tp[15]} = $tp[0];
  $tp[2] *= 5;
  $tlist[$i] = \@tp;
}
for ($i = 11; $i < 33; $i++) {#load specials
  chomp ($x = <IFILE>);  
  my @tp = split(/,/,$x);
  for ($j = 1; $j < 6; $j++) {
    $tp[$j] =~ s/ //g;
  }
  $tlist[$i] = \@tp;
}
$tshort{"yes"} = "yes";
$tshort{"no"} = "no";

##this is great
sub swapt  {
  my $a = shift;
  my $b = shift;
  $tlist[33] = $tlist[$a];
  $tlist[$a] = $tlist[$b];
  $tlist[$b] = $tlist[33];
  $tlist[33] = $tlist[$a+11];
  $tlist[$a+11] = $tlist[$b+11];
  $tlist[$b+11] = $tlist[33];
  $tlist[33] = $tlist[$a+22];
  $tlist[$a+22] = $tlist[$b+22];
  $tlist[$b+22] = $tlist[33];
}
swapt(0,7);
swapt(1,7);
swapt(2,3);
swapt(5,9);
swapt(6,9);
swapt(7,10);
swapt(8,10);
swapt(9,10);

open (TT, "ttemp.txt") || die "There needs to be a ttemp.txt in the current directory.";
open TOUT, ">", "terrain.ruleset";
$x = "";
while (chomp($x = <TT>) && !($x eq "####")) {
  print TOUT "$x\n";
}

for ($i = 0; $i < 11; $i++) {
  $name3 = $tlist[$i][0];
  $name3 =~ tr /[A-Z] /[a-z]_/;
  $tlist[$i][6] = $tshort{$tlist[$i][6]};
  $tlist[$i][10] = $tshort{$tlist[$i][10]};
  $tlist[$i][14] = $tshort{$tlist[$i][14]};
  print TOUT <<OUTTERR;
[terrain_$name3]
terrain_name		= _("$tlist[$i][0]")
movement_cost		= $tlist[$i][1]
defense_bonus		= $tlist[$i][2]
food			= $tlist[$i][3]
shield			= $tlist[$i][4]
trade			= $tlist[$i][5]
special_1_name		= _("$tlist[$i+11][0]")
food_special_1		= $tlist[$i+11][3]
shield_special_1	= $tlist[$i+11][4]
trade_special_1		= $tlist[$i+11][5]
special_2_name		= _("$tlist[$i+22][0]")
food_special_2		= $tlist[$i+22][3]
shield_special_2	= $tlist[$i+22][4]
trade_special_2		= $tlist[$i+22][5]
irrigation_result	= "$tlist[$i][6]"
irrigation_food_incr	= $tlist[$i][7]
irrigation_time		= $tlist[$i][8]
mining_result		= "$tlist[$i][10]"
mining_shield_incr	= $tlist[$i][11]
mining_time		= $tlist[$i][12]
transform_result	= "$tlist[$i][14]"
OUTTERR
  $x = " ";
  while (!($x eq "") && chomp($x = <TT>)) {
    print TOUT "$x\n";
  }
  if ($i eq 8) {
    $x = " ";
    while (!($x eq "") && chomp($x = <TT>)) {
      print TOUT "$x\n";
    }
  }
}
close TOUT;
close TT;






@nationflags = qw(f.rome f.iraq_old f.germany f.egypt f.usa f.greece
f.india f.russia f.south_africa f.france f.mexico f.china f.england
f.mongolia f.wales f.japan f.norway f.spain f.iran f.tunisia f.cheyenne);
$nations = 1;
open (CTN, "city.txt") || ($nations = 0); ##don't bother with the rest then
if ($nations = 0) {
  close IFILE;
  exit(0);
}
open NOUT, ">", "nations.ruleset";
print NOUT <<OUTNAT; 
[datafile]
description="Civmod nations data for Freeciv"
options="1.9"

OUTNAT

while (chomp ($x = <IFILE>) && !($x eq "\@LEADERS")) {}
$nations = 0;
while (chomp ($x = <IFILE>) && !($x eq "") && !($x eq ";")) {
  $x =~ s/, +/,/g;
  my @nationset = split(/,/,$x);
  $name2 = $nationset[6];
  $name2 =~ tr/[A-Z] /[a-z]_/;
  $nstyle = "Ancient" if ($nationset[4] eq 0);
  $nstyle = "Classical" if ($nationset[4] eq 1);
  $nstyle = "Oriental" if ($nationset[4] eq 2);
  $nstyle = "European" if ($nationset[4] eq 3);
  $nationset[7]++;
  $nationset[8]++;
  $nationset[9]++;
  print NOUT <<OUTNAT;  
[nation_$name2]

name=_("$nationset[6]")
plural=_("?plural:$nationset[5]")
leader="$nationset[0]", "$nationset[1]"
leader_sex="Male", "Female"
flag="$nationflags[$nations]"
flag_alt="-"
city_style = "$nstyle"

attack=$nationset[7]
expand=$nationset[8]
civilized=$nationset[9]
advisors=100,100,100,100,100,100,100
init_techs=""
tech_goals = "$short{'Rep'}", "$short{'Tra'}", "$short{'Eng'}", "$short{'Dem'}",
             "$short{'RR'}", "$short{'Exp'}", "$short{'Aut'}"
wonder="$bname[$impnum - 9]"
government="$gname[6]"

OUTNAT
  if (!($nationset[10] eq "")) {
    print NOUT 'ruler_titles = { "government",      "male_title",      "female_title"';
    print NOUT "\n";
    $i = 10;
    while (!($nationset[$i] eq "")) {
      print NOUT <<OUTNAT;
                 "$gname[$nationset[$i]]", _("$nationset[$i + 1]"), _("$nationset[$i + 2]")
OUTNAT
      $i += 3;
    }
    print NOUT "               }\n\n";
  }
  seek CTN, 0, 0;
  $name2 = $nationset[5];
  $name2 =~ tr/[a-z]/[A-Z]/; 
  print NOUT "cities = \n";
  while (chomp ($x = <CTN>) && !($x =~ /\@$name2/)) {}
  $i = 0;
  while (chomp ($x = <CTN>) && !($x eq "\@STOP")) {
    if ($i > 0) {
      print NOUT ",\n";
    } else {
      $i = 1;
    }
    print NOUT "  \"$x\"";
  }
  print NOUT '  "Nowhere"' if ($i eq 0);
  print NOUT "\n\n";

  $nations++;
}

print NOUT "[misc]\n\n";
print NOUT "cities = \n";
while (chomp ($x = <CTN>) && !($x =~ /\@EXTRA/)) {}
$i = 0;
while (chomp ($x = <CTN>) && !($x eq "\@STOP")) {
  if ($i > 0) {
    print NOUT ",\n";
  } else {
    $i = 1;
  }
  print NOUT "  \"$x\"";
}
print NOUT "\n\n";




print NOUT <<OUTNAT; 
[nation_barbarian]

name=_("Barbarian")
plural=_("?plural:Barbarians")
leader="Genseric", "Alaric", "Teodoric", "Herman"
leader_sex="Male", "Male", "Male", "Male"
flag="f.barbarian"
flag_alt = "-"
city_style = "European"

attack=0
expand=0
civilized=0
advisors=100,100,100,100,100,100,100
init_techs=""

tech_goals = "$short{'War'}"
wonder = "$bname[$impnum - 3]"
government = "$gname[1]"

cities = 
OUTNAT

while (chomp ($x = <CTN>) && !($x =~ /\@BARBARIANS/)) {}
$x = <CTN>; # what does this number mean?
$i = 0;
while (chomp ($x = <CTN>) && !($x eq "\@STOP") && !($x eq "")) {
  if ($i > 0) {
    print NOUT ",\n";
  } else {
    $i = 1;
  }
  print NOUT "  \"$x\"";
}
print NOUT "\n\n";




close NOUT;
close CTN;

close IFILE;
 